#include <stdio.h>
#include <stdint.h>
#include "randombytes.h"

#define MSGBYTES 59
#define KEYBYTES 16
#define TAGBYTES 16

int poly1305_c(uint8_t *out,
               const uint8_t *in,
               unsigned long long inlen,
               const uint8_t *k);

int poly1305_jasmin(uint8_t *out,
                    const uint8_t *in,
                    unsigned long long inlen,
                    const uint8_t *k);

int poly1305_verify_c(const uint8_t *h, 
                      const uint8_t *in, 
                      unsigned long long inlen, 
                      const uint8_t *k);

int poly1305_verify_jasmin(const uint8_t *h, 
                           const uint8_t *in, 
                           unsigned long long inlen, 
                           const uint8_t *k);



int main(void) {
  uint8_t msg[MSGBYTES];
  uint8_t key[KEYBYTES];
  uint8_t tag0[TAGBYTES], tag1[TAGBYTES];
  int i;

  /* Sample random message */
  randombytes(msg, MSGBYTES);

  /* Sample random key */
  randombytes(key, KEYBYTES);

  /* Compute tag using C implemenation of onetimeauth */
  poly1305_c(tag0, msg, MSGBYTES, key);

  /* Compute tag using jasmin implemenation of onetimeauth */
  poly1305_jasmin(tag1, msg, MSGBYTES, key);

  /* Check that both implementations compute the same tag */
  for(i=0;i<TAGBYTES;i++)
    if(tag0[i] != tag1[i])
      printf("Error: C and jasmin implementations compute different tag\n");

  /* Use C verify implementation to check valid tag */
  if(poly1305_verify_c(tag0, msg, MSGBYTES, key))
    printf("Error: C implementation of verify fails with valid tag\n");
  
  /* Use C verify implementation to check some invalid tags */
  for(i=0;i<TAGBYTES;i++) {
    tag0[i] ^= 128;
    if(!poly1305_verify_c(tag0, msg, MSGBYTES, key))
      printf("Error: C implementation of verify passes with invalid tag\n");
    tag0[i] ^= 128;
  }

  /* Use jasmin verify implementation to check valid tag */
  if(poly1305_verify_jasmin(tag0, msg, MSGBYTES, key))
    printf("Error: jasmin implementation of verify fails with valid tag\n");
  
  /* Use jasmin verify implementation to check some invalid tags */
  for(i=0;i<TAGBYTES;i++) {
    tag0[i] ^= 128;
    if(!poly1305_verify_jasmin(tag0, msg, MSGBYTES, key))
      printf("Error: jasmin implementation of verify passes with invalid tag\n");
    tag0[i] ^= 128;
  }

  return 0;
}
